/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "MavlinkModule.h"

#include <Core/MessageBroker/MessageBroker.h>

#include <QDebug>
#include <QSerialPortInfo>

#include "Components/ContextMenuSeparator/ContextMenuSeparator.h"
#include "Modules/SkywardHubStrings.h"

MavlinkModule::MavlinkModule(QWidget *parent)
    : DefaultModule(parent), mavlinkReader(this), serialPort(this)
{
    setupUi();
    defaultContextMenuSetup();
    initializeSerialPortView();

    connect(&mavlinkReader, &MavlinkReader::msgReceived, this,
            &MavlinkModule::onMsgReceived);
    connect(&linkQualityTimer, &QTimer::timeout, this,
            &MavlinkModule::onLinkQualityTimerTick);

    mavlinkCommandAdapter.setSerialPort(&serialPort);

    getCore()->getMessageBroker()->subscribe(
        Filter::fromString(SkywardHubStrings::commandsTopic + "/*"), this,
        [this](const Message &message, const Filter &filter)
        { onCommandReceived(message); });
}

MavlinkModule::~MavlinkModule() { onStopClicked(); }

QWidget *MavlinkModule::toWidget() { return this; }

XmlObject MavlinkModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::MAVLINK));
    obj.addAttribute("serial_port", portsComboBox->currentText());
    obj.addAttribute("baudrate", baudrateComboBox->currentData().toInt());
    obj.addAttribute("log_file", logCheckBox->isChecked() ? 1 : 0);
    return obj;
}

void MavlinkModule::fromXmlObject(const XmlObject &obj)
{
    if (obj.getObjectName() == getName(ModuleId::MAVLINK))
    {
        QString serialPort = obj.getAttribute("serial_port");
        int baudrate;
        obj.getAttribute("baudrate", baudrate);
        int logFile;
        obj.getAttribute("log_file", logFile);

        portsComboBox->setCurrentText(serialPort);
        baudrateComboBox->setCurrentText(QString::number(baudrate));
        logCheckBox->setChecked(logFile != 0);
    }
}

void MavlinkModule::closePort()
{
    if (serialPort.isOpen())
        serialPort.close();
}

void MavlinkModule::onStartClicked()
{
    if (startReadingOnSerialPort())
    {
        linkQualityTimer.start(linkQualityPeriod);

        mavlinkReader.setSerialPort(&serialPort);
        mavlinkCommandAdapter.setSerialPort(&serialPort);

        // TODO: Allow the checkbox to start and stop the logging
        if (logCheckBox->isChecked())
            mavlinkReader.openLogFile();

        mavlinkReader.startReading();
    }
    else
    {
        error("Mavlink", "Unable to open selected serial port.");
        startSerialStreamToggleButton.setState(false);
    }
}

void MavlinkModule::onStopClicked()
{
    linkQualityTimer.stop();
    mavlinkReader.stopReading();
    closePort();
}

void MavlinkModule::onMsgReceived(const Message &msg)
{
    msgArrived++;
    getCore()->getMessageBroker()->publish(msg);
}

void MavlinkModule::onStartStreamToggled(bool state)
{
    if (state)
        onStartClicked();
    else
        onStopClicked();
}

void MavlinkModule::onLinkQualityTimerTick()
{
    float ratio = (float)msgArrived / linkQualityPeriod * 1000.0;
    msgArrived  = 0;
    rateLabel->setText("Rate: " + QString::number(ratio) + " msg/s");
}

void MavlinkModule::onOpenLogFolderClick()
{
    QDir dir(SkywardHubStrings::defaultLogsFolder);
    if (dir.exists())
    {
        QDesktopServices::openUrl(
            QUrl::fromLocalFile(SkywardHubStrings::defaultLogsFolder));
    }
    else
    {
        warning(tr("Mavlink"), tr("The log folder does not exist yet. It will "
                                  "be created when opening the serial port."));
    }
}

void MavlinkModule::onCommandReceived(const Message &msg)
{
    mavlink_message_t encoded_mvl_msg;
    if (!mavlinkCommandAdapter.encodeCommand(msg, encoded_mvl_msg))
    {
        error(tr("Mavlink"),
              tr("Cannot encode command into a valid mavlink message."));
    }
    else if (!serialPort.isOpen())
    {
        error(tr("Mavlink"), "Cannot send message: serial port is closed.");
    }
    else
    {
        mavlinkCommandAdapter.send(encoded_mvl_msg);

        Message confirmationMsg(
            Topic((QString)SkywardHubStrings::logCommandsTopic));

        auto nameField = Field(msg.getTopic().toString().replace(
            SkywardHubStrings::commandsTopic + "/", ""));
        confirmationMsg.setField("name", nameField);

        auto idField = Field((uint64_t)encoded_mvl_msg.msgid);
        confirmationMsg.setField("message_id", idField);

        auto seqField = Field((uint64_t)encoded_mvl_msg.seq);
        confirmationMsg.setField("sequence_number", seqField);

        getCore()->getMessageBroker()->publish(confirmationMsg);
    }
}

void MavlinkModule::setupUi()
{
    QHBoxLayout *outerLayout = new QHBoxLayout;
    outerLayout->setContentsMargins(6, 0, 6, 0);

    QLabel *portsLabel = new QLabel("Available ports:");
    portsLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(portsLabel);

    portsComboBox = new QComboBox;
    portsComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(portsComboBox);

    QLabel *baudrateLabel = new QLabel("Baudrate:");
    baudrateLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(baudrateLabel);

    baudrateComboBox = new QComboBox;
    baudrateComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(baudrateComboBox);

    ToggleButton *startToggleButton = new ToggleButton;
    startToggleButton->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(startToggleButton);
    connect(startToggleButton, &ToggleButton::toggled, this,
            &MavlinkModule::onStartStreamToggled);

    outerLayout->addStretch();

    rateLabel = new QLabel("Rate: 0 msg/s");
    rateLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(rateLabel);

    logCheckBox = new QCheckBox("Enable log");
    logCheckBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(logCheckBox);

    QPushButton *openLogFolderButton = new QPushButton("Log folder");
    openLogFolderButton->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    outerLayout->addWidget(openLogFolderButton);
    connect(openLogFolderButton, &QPushButton::clicked, this,
            &MavlinkModule::onOpenLogFolderClick);

    setLayout(outerLayout);
}

void MavlinkModule::initializeSerialPortView()
{
    const auto serialPortInfos = QSerialPortInfo::availablePorts();
    for (auto serialPortInfo : serialPortInfos)
        portsComboBox->addItem(serialPortInfo.portName());

    baudrateComboBox->addItem("115200", QSerialPort::Baud115200);
    baudrateComboBox->addItem("19200", QSerialPort::Baud19200);
    baudrateComboBox->addItem("9600", QSerialPort::Baud9600);
}

bool MavlinkModule::startReadingOnSerialPort()
{
    QString portName = portsComboBox->currentText();
    bool baudrateOk;
    int baudRate = baudrateComboBox->currentData().toInt(&baudrateOk);

    // Check if the parameters are ok
    if (!baudrateOk)
        return false;

    // The serial port should not be already open
    if (serialPort.isOpen())
        return true;

    // Open the serial port
    QSerialPortInfo port(portName);
    serialPort.setPort(port);
    serialPort.setBaudRate(baudRate);
    serialPort.setDataBits(QSerialPort::Data8);
    serialPort.setParity(QSerialPort::NoParity);
    serialPort.setStopBits(QSerialPort::OneStop);
    return serialPort.open(QIODevice::ReadWrite);
}
