/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Deployer.h"

#include <QDir>
#include <QFile>
#include <QMessageBox>
#include <QTextStream>

Deployer::Deployer() {}

bool Deployer::deploy(const QList<QString>& activeModules)
{
    setupModulesFiles(activeModules);

    if (!deployerPicker.pickDirectory())
    {
        return false;
    }

    proFilePath = deployerPicker.getProFilePathFolder();
    deployDir   = deployerPicker.getDeploymentFolder();

    if (!isDeployConfirmed())
        return false;

    makeDeployDir();
    copyPath(getSourceDir(), getNewSourceDir());
    produceNewProFile();
    produceNewModuleListFile();

    system(qPrintable("cmd.exe /A /C " + writeBatFile()));

    return showResultMessage();
}

QString Deployer::getDeployReleaseFolder() const
{
    return deployDir + "/" + newCopyReleaseFolderName;
}

void Deployer::setupModulesFiles(const QList<QString>& activeModules)
{
    modulesFiles.clear();
    modulesFiles.append("SOURCES");
    modulesFiles.append("HEADERS");
    modulesFiles.append("FORMS");

    /*
     * This are the mandatory files to include in the build.
     * If you want to change this list, probablly you must edit also the
     * ModulesManager in the Core.
     *
     * The files are listed in order of how much they are deep-rooted (radicati)
     * in the system (so the first files are the most difficul to replace while
     * the lastone is the easyest)
     */

    modulesFiles.append("Core/");
    modulesFiles.append("moduleinfo");
    modulesFiles.append("moduleslist");
    modulesFiles.append("SkywardHubStrings");
    modulesFiles.append("main.cpp");
    modulesFiles.append("Modules/Empty/");
    modulesFiles.append("Modules/DefaultModule/");
    modulesFiles.append("Modules/Splitter/");
    modulesFiles.append("Modules/MainWindow/");
    modulesFiles.append("Components/");

    modulesFiles.append(activeModules);
}

// _________________________________ PRO FILE
// ______________________________________________________

void Deployer::produceNewProFile()
{
    QString filePath          = getNewProFilePath();
    QList<QString> proFile    = loadFile(filePath);
    QList<QString> newProFile = makeNewProFile(proFile);
    writeFile(newProFile, filePath);
}

QList<QString> Deployer::makeNewProFile(const QList<QString>& oldProFile) const
{
    QList<QString> newProFile;

    bool copy = true;
    for (QString line : oldProFile)
    {
        if (line.contains(proFileEndCopyTag))
        {
            copy = false;
        }
        else if (line.contains(proFileRestartCopyTag))
        {
            copy = true;
        }

        if (copy || line.isEmpty() || checkIfFilesIsActive(line))
        {
            newProFile.append(line);
        }
    }

    return newProFile;
}

bool Deployer::checkIfFilesIsActive(const QString& line) const
{
    for (QString file : modulesFiles)
    {
        if (line.contains(file))
        {
            return true;
        }
    }
    return false;
}

// _____________________________ modules list.cpp FILE
// ______________________________________________________

void Deployer::produceNewModuleListFile()
{
    QString filePath              = getModuleListFilePath();
    QList<QString> oldModulesFile = loadFile(filePath);
    QList<QString> newModulesFile = makeNewModulesListFile(oldModulesFile);
    writeFile(newModulesFile, filePath);
}

QList<QString> Deployer::makeNewModulesListFile(
    const QList<QString>& oldModulesListFile) const
{
    QList<QString> newFile;

    for (QString line : oldModulesListFile)
    {
        if (line.contains("#include"))
        {
            if (checkIfFilesIsActive(line))
            {
                newFile.append(line);
            }
        }
        else
        {
            newFile.append(line);
        }
    }

    return newFile;
}

// _____________________________ BAT FILE
// ___________________________________________________________________

QString Deployer::writeBatFile() const
{
    QString batfile_header = "set PATH=" + mingw81_64_bin + ";" +
                             mingw810_64_bin +
                             ";%PATH%\n"
                             "cd /D " +
                             mingw81_64_bin + "\n";
    QString batfile_core = "cd " + deployDir +
                           "\n"
                           "qmake  -config release " +
                           getNewProFilePath() +
                           "\n"
                           "mingw32-make.exe -f Makefile.Release\n"
                           "mingw32-make.exe -f Makefile.Release clean\n"
                           "windeployqt release/.";

    QString batfile_path = getNewSourceDir() + "\\deployer.bat";
    QFile file(batfile_path);
    if (file.open(QIODevice::WriteOnly))
    {
        QTextStream(&file) << batfile_header << batfile_core;
        file.close();
        return batfile_path;
    }
    return "";
}

// __________________________________________________________________________________________________________

bool Deployer::isDeployCompletedSuccessfully()
{
    QDir dir(getDeployReleaseFolder());
    if (dir.exists() && !dir.isEmpty())
        return true;

    return false;
}

bool Deployer::showResultMessage()
{
    QMessageBox confirmMsg;

    if (isDeployCompletedSuccessfully())
    {
        confirmMsg.setText("Deploy completed successfully");
        confirmMsg.exec();
        return true;
    }
    else
    {
        QString text = "Deployment Error";
        QString infoMsg =
            "Something went wrong. \n\nCheck if the new modules specify "
            "correctly all the needed files.";
        confirmMsg.warning(nullptr, text, infoMsg);
        confirmMsg.setInformativeText(infoMsg);
        return false;
    }
}

bool Deployer::isDeployConfirmed()
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Deploy");
    QString infoMsg = "Do you want to deploy the following project: \n" +
                      proFilePath + "\n\nIn the following directory: \n" +
                      deployDir;
    msgBox.setInformativeText(infoMsg);
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
    msgBox.setDefaultButton(QMessageBox::Yes);

    if (msgBox.exec() == QMessageBox::Yes)
        return true;
    return false;
}

void Deployer::copyPath(const QString& src, const QString& dst)
{
    QDir dir(src);
    if (!dir.exists())
        return;

    foreach (QString d, dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot))
    {
        QString dst_path = dst + QDir::separator() + d;
        dir.mkpath(dst_path);
        copyPath(src + QDir::separator() + d, dst_path);
    }

    foreach (QString f, dir.entryList(QDir::Files))
    {
        QFile::copy(src + QDir::separator() + f, dst + QDir::separator() + f);
    }
}

QList<QString> Deployer::loadFile(const QString& path)
{
    QList<QString> stringList;
    QFile textFile(path);
    if (textFile.open(QIODevice::ReadOnly))
    {
        QTextStream textStream(&textFile);
        while (!textStream.atEnd())
        {
            stringList.append(textStream.readLine());
        }
        textFile.close();
    }
    return stringList;
}

QString Deployer::getProFileName() const
{
    return QFileInfo(proFilePath).fileName();
}

QString Deployer::getSourceDir() const
{
    return QFileInfo(proFilePath).dir().absolutePath();
}

QString Deployer::getNewSourceDir() const
{
    return deployDir + "/" + newCopySourceFolderName;
}

QString Deployer::getNewProFilePath() const
{
    return getNewSourceDir() + "/" + getProFileName();
}

QString Deployer::getModuleListFilePath() const
{
    return getNewSourceDir() + relativeModuleFilesPath;
}

void Deployer::makeDeployDir() const
{
    QDir newDir(deployDir);
    newDir.mkdir(newCopySourceFolderName);
    //    newDir.mkdir(newCopyReleaseFolderName);
}

bool Deployer::writeFile(const QList<QString>& text, const QString& path)
{
    bool result = false;
    QFile file(path);
    if (file.open(QIODevice::WriteOnly))
    {
        QTextStream writer(&file);
        for (QString line : text)
        {
            writer << line << '\n';
        }
        file.close();
        result = true;
    }
    return result;
}
