/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "SkywardHubModule.h"

#include <Core/ModulesManager/ModulesManager.h>

#include <QDir>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>

#include "Deployer.h"
#include "Modules/MainWindow/SkywardHubMainWindow.h"
#include "Modules/SkywardHubStrings.h"
#include "PrefabDialog.h"
#include "PrefabViewElement.h"
#include "QMessageBox"
#include "ui_SkywardHubModule.h"

SkywardHubModule::SkywardHubModule(QWidget *parent)
    : DefaultModule(parent), ui(new Ui::SkywardHubModule)
{
    ui->setupUi(this);

    defaultContextMenuSetup();
    fillCompleteModulesList();
    addEmptyPrefab();
    loadPrefabs();

    connect(ui->pushButton_deploy, &QPushButton::clicked, this,
            [this]() { onPrefabDeployRequested(selectedElement); });
}

SkywardHubModule::~SkywardHubModule()
{
    clearPrefab();
    delete emptyPrefab;
    emptyPrefab = nullptr;
    for (QCheckBox *c : completeModule_checkboxList)
    {
        delete c;
    }
    delete ui;
}

void SkywardHubModule::fillCompleteModulesList()
{
    QList<QString> completeModuleNamesList =
        ModulesManager::getModulesNamesList();

    for (QString s : completeModuleNamesList)
    {
        QCheckBox *cBox = createPanelsCheckbox(s);
        insertCheckBox(cBox);
        connect(cBox, &QCheckBox::clicked, this,
                [this, cBox]() { onCheckBoxClicked(cBox); });
    }
}

void SkywardHubModule::loadPrefabs()
{
    clearPrefab();
    nextColumn = 0;
    nextRow    = 0;

    QString prefabsFolder = SkywardHubStrings::defaultPrefabsFolder;

    QDir directory(prefabsFolder);
    QStringList prefabFiles =
        directory.entryList(QStringList() << "*.xml", QDir::Files);
    for (QString prefabName : prefabFiles)
    {
        XmlObject xml;
        xml.loadFromFile(prefabsFolder + prefabName);
        createAndInsertPrefabFromXml(xml, prefabsFolder + prefabName);
    }
}

void SkywardHubModule::updateActiveCheckboxList(PrefabViewElement *element)
{
    if (element == nullptr)
        return;

    for (QCheckBox *cBox : completeModule_checkboxList)
    {
        bool isModuleActive = element->containsModule(cBox->text());
        cBox->setChecked(isModuleActive);
    }
}

void SkywardHubModule::initUsedModules(PrefabViewElement *element,
                                       const XmlObject &root) const
{
    if (element == nullptr)
        return;

    // Set as active the used modules
    element->addActiveModuleName(root.getObjectName());
    for (int i = 0; i < root.childCount(); i++)
    {
        initUsedModules(element, root.childAt(i));
    }
}

void SkywardHubModule::addEmptyPrefab()
{
    Module *module = getCore()->getModulesManager()->instantiateDefaultModule();
    if (module != nullptr)
    {
        XmlObject xmlconfig(SkywardHubStrings::configurationNameValue);
        xmlconfig.addAttribute(SkywardHubStrings::configurationNameAttribute,
                               "Launch Empty Configuration");
        XmlObject emptyXml = module->toXmlObject();
        xmlconfig.addChild(emptyXml);
        PrefabViewElement *prefabView = createNewPrefab(xmlconfig);
        if (prefabView != nullptr)
        {
            initUsedModules(prefabView, xmlconfig);
            prefabView->setDeleteButtonVisibility(false);
            prefabView->setIconButtonVisibility(false);
            prefabView->setLaunchButtonText("+");
            prefabView->setDesctiption(
                "Create your custom configuration and then save it (Right "
                "Click -> HubMenu -> Save)");
            emptyPrefab = prefabView;
            connect(prefabView, &PrefabViewElement::launchRequested, this,
                    &SkywardHubModule::onPrefabLaunchRequested);
            ui->layout_addNewConfig->addWidget(prefabView);
        }
    }
}

void SkywardHubModule::createAndInsertPrefabFromXml(XmlObject &xml,
                                                    const QString &filePath)
{
    PrefabViewElement *prefabView = createNewPrefab(xml);
    if (prefabView != nullptr)
    {
        prefabView->setFilePath(filePath);
        initUsedModules(prefabView, xml);
        addPrefab(prefabView);
        insertPrefabInUi(prefabView);
    }
}

QCheckBox *SkywardHubModule::createPanelsCheckbox(const QString &text)
{
    QString check_stylesheet =
        ""
        "QCheckBox"
        "{"
        "background-color: none;"
        "border: 3px solid transparent;"
        "color: #00bfff;"
        "}"
        "QCheckBox:hover"
        "{"
        "background-color: #474747;"
        //                "border: 3px solid #00bfff;"
        "border-top: 1px solid none;border-right: 3px solid #00bfff; "
        "border-bottom: 1px solid none; border-left:  3px solid #00bfff;"
        "color: #00bfff;"
        "}"
        "QCheckBox:checked {  background-color: QLinearGradient( x1: 0, y1: 0, "
        "x2: 1, y2: 0, stop: 0 #506987, stop: 1  #00739a);"
        //                               "border: 2px solid #00bfff;" /*#3f839a
        //                               #00739a*/
        "border-top: none;border-right: 3px solid #00bfff; border-bottom: "
        "none; border-left:  3px solid #00bfff;}";

    QCheckBox *checkBox = new QCheckBox(text);
    checkBox->setStyleSheet(check_stylesheet);
    return checkBox;
}

void SkywardHubModule::insertCheckBox(QCheckBox *checkBox)
{
    ui->modules_layout->insertWidget(0, checkBox);
    completeModule_checkboxList.append(checkBox);
}

void SkywardHubModule::addPrefab(PrefabViewElement *element)
{
    prefabs.append(element);
    connect(element, &PrefabViewElement::launchRequested, this,
            &SkywardHubModule::onPrefabLaunchRequested);
    connect(element, &PrefabViewElement::deleteRequested, this,
            &SkywardHubModule::onPrefabDeleteRequested);
    connect(element, &PrefabViewElement::selected, this,
            &SkywardHubModule::onPrefabSelected);
}

void SkywardHubModule::insertPrefabInUi(PrefabViewElement *element)
{
    // Insert the prefabView in the correct column

    if (nextColumn == maxColum)
    {
        nextColumn = 0;
        nextRow++;
    }
    ui->gridLayout_prefab->addWidget(element, nextRow, nextColumn);
    nextColumn++;
}

void SkywardHubModule::clearPrefab()
{
    for (PrefabViewElement *p : prefabs)
    {
        delete p;
    }
    prefabs.clear();
    onPrefabSelected(nullptr);
}

PrefabViewElement *SkywardHubModule::createNewPrefab(XmlObject &xmlPrefab)
{
    if (xmlPrefab.getObjectName() == SkywardHubStrings::configurationNameValue)
    {
        QString prefName = xmlPrefab.getAttribute(
            SkywardHubStrings::configurationNameAttribute);
        if (!prefName.isEmpty())
        {
            PrefabViewElement *prefabView = new PrefabViewElement(prefName);
            prefabView->setXml(xmlPrefab);
            return prefabView;
        }
    }
    return nullptr;
}

void SkywardHubModule::dialogRequested()
{
    PrefabDialog dialog(this);
    // connect(&dialog, &PrefabDialog::newPrefabCreated, this,
    // &SkywardHubModule::createNewPrefab);
    dialog.exec();
}

void SkywardHubModule::onPrefabSelected(PrefabViewElement *element)
{
    selectedElement = element;
    updateActiveCheckboxList(selectedElement);
    ui->plainTextEdit->clear();

    if (element != nullptr)
    {
        ui->groupBox_selectedModule->setTitle(element->getName());
        ui->plainTextEdit->insertPlainText(element->getXml().toXml());
    }
}

void SkywardHubModule::onPrefabLaunchRequested(PrefabViewElement *element)
{
    if (element != nullptr)
    {
        XmlObject xml = element->getXml();
        getCore()->getModulesManager()->setModules(
            getCore()->getModulesManager()->loadModuleFromXml(xml));
    }
}

void SkywardHubModule::onPrefabDeployRequested(PrefabViewElement *element)
{
    if (element == nullptr)
    {
        QMessageBox msgBox;
        msgBox.setText("No configuration selected.");
        msgBox.exec();
        return;
    }

    Deployer deployer;

    QList<QString> modulesFiles;
    QList<ModuleInfo> fullModuleList =
        getCore()->getModulesManager()->getModulesInfo();

    for (ModuleInfo mInfo : fullModuleList)
    {
        if (element->containsModule(mInfo.getModuleName()))
        {
            modulesFiles.append(mInfo.getModuleSourceFiles());
        }
    }

    bool deployResult = deployer.deploy(modulesFiles);

    if (deployResult == false)
        return;

    // Put the prefab config as default config
    QString configPath = deployer.getDeployReleaseFolder() + "/" +
                         SkywardHubStrings::defaultConfigurationFolder;
    if (!QDir(configPath).exists())
    {
        QDir().mkdir(configPath);
    }
    configPath += "/" + SkywardHubStrings::defaultPrefabsFolderName;
    if (!QDir(configPath).exists())
    {
        QDir().mkdir(configPath);
    }
    element->getXml().writeToFile(
        configPath + "/" + SkywardHubStrings::defaultConfigurationFileName);
}

void SkywardHubModule::onPrefabDeleteRequested(PrefabViewElement *element)
{
    if (element == nullptr)
        return;

    QMessageBox msgBox;
    QString title = "Delete configuration";
    QString text  = "Do you want to delete the following configuration?\n" +
                   element->getFilePath();
    if (msgBox.question(this, title, text) == QMessageBox::Yes)
    {
        QFile file(element->getFilePath());
        if (file.exists())
        {
            file.remove();
            loadPrefabs();
        }
    }
}

void SkywardHubModule::onCheckBoxClicked(QCheckBox *cBox)
{
    if (cBox == nullptr || selectedElement == nullptr)
        return;

    if (cBox->isChecked())
    {
        selectedElement->addActiveModuleName(cBox->text());
    }
    else
    {
        selectedElement->removeActiveModuleName(cBox->text());
    }
}

QWidget *SkywardHubModule::toWidget() { return this; }

XmlObject SkywardHubModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::SKYWARDHUB));
}

void SkywardHubModule::fromXmlObject(const XmlObject &xmlObject)
{
    Q_UNUSED(xmlObject);
}
