/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Splitter.h"

#include <Components/ContextMenuSeparator/ContextMenuSeparator.h>
#include <Core/ModulesManager/ModulesManager.h>

#include <QDebug>
#include <QLabel>

Splitter::Splitter(Qt::Orientation orientation, QWidget* parent)
    : DefaultModule(parent)
{
    setupUi(orientation);
    defaultContextMenuSetup();
}

QWidget* Splitter::toWidget() { return this; }

XmlObject Splitter::toXmlObject()
{
    XmlObject obj(getName(ModuleId::SPLITTER));

    // Orientation
    if (splitter->orientation() == Qt::Horizontal)
        obj.addAttribute("orientation", "horizontal");
    else
        obj.addAttribute("orientation", "vertical");

    QString sizes;
    for (int i = 0; i < splitter->sizes().size(); i++)
    {
        sizes += QString::number(splitter->sizes()[i]);

        if (i < splitter->sizes().size() - 1)
            sizes += ",";
    }

    obj.addAttribute("sizes", sizes);

    for (int i = 0; i < splitter->count(); i++)
    {
        DefaultModule* module =
            qobject_cast<DefaultModule*>(splitter->widget(i));

        if (module)
            obj.addChild(module->toXmlObject());
    }

    return obj;
}

void Splitter::fromXmlObject(const XmlObject& obj)
{
    QString orientation = obj.getAttribute("orientation");
    if (orientation == "vertical")
        splitter->setOrientation(Qt::Vertical);
    else
        splitter->setOrientation(Qt::Horizontal);
    QList<int> intSizes;
    for (QString size : obj.getAttribute("sizes").split(","))
        intSizes.append(size.toInt());

    for (int i = 0; i < obj.childCount(); i++)
    {
        XmlObject xmlChild = obj.childAt(i);
        Module* module =
            getCore()->getModulesManager()->instantiateModuleByName(
                xmlChild.getObjectName());

        if (module)
            module->fromXmlObject(xmlChild);
        else
            module = getCore()->getModulesManager()->instantiateDefaultModule();

        addModule(module, i);
    }

    splitter->setSizes(intSizes);
}

void Splitter::addModule(Module* module)
{
    int count = splitter->count();
    addModule(module, count);
}

void Splitter::addModule(Module* module, int position)
{
    connect(module->getEventHandler(), &EventHandler::replaceMeWith, this,
            &Splitter::replace);
    connect(module->getEventHandler(), &EventHandler::contextMenuRequest, this,
            &Splitter::onSkywardHubContextMenuRequested);

    if (position < splitter->count())
        delete splitter->widget(position);

    splitter->insertWidget(position, qobject_cast<QWidget*>(module));
}

void Splitter::replace(Module* oldModule, Module* newModule)
{
    if (oldModule && newModule)
        addModule(newModule, splitter->indexOf(oldModule->toWidget()));
}

Qt::Orientation Splitter::getOrientation() const
{
    return splitter->orientation();
}

void Splitter::setOrientation(Qt::Orientation orientation)
{
    splitter->setOrientation(orientation);
}

void Splitter::swapOrientation()
{
    if (getOrientation() == Qt::Horizontal)
        splitter->setOrientation(Qt::Vertical);
    else
        splitter->setOrientation(Qt::Horizontal);
}

void Splitter::addCustomActionsToMenu()
{
    showMenuActionsInSeparatedMenu("Splitter");
    QAction* split = new QAction("Split");
    connect(split, &QAction::triggered, this,
            [this]() {
                addModule(
                    getCore()->getModulesManager()->instantiateDefaultModule());
            });

    QAction* changeOrientation = new QAction("Swap Orientation");
    connect(changeOrientation, &QAction::triggered, this,
            [this]() { this->swapOrientation(); });

    addActionToMenu(split);
    addActionToMenu(changeOrientation);
}

void Splitter::setupUi(Qt::Orientation orientation)
{
    QVBoxLayout* outerLayout = new QVBoxLayout;
    outerLayout->setContentsMargins(0, 0, 0, 0);

    splitter = new QSplitter();
    splitter->setOrientation(orientation);
    splitter->setChildrenCollapsible(false);
    splitter->setHandleWidth(0);
    outerLayout->addWidget(splitter);

    // By default add two default modules
    addModule(getCore()->getModulesManager()->instantiateDefaultModule());
    addModule(getCore()->getModulesManager()->instantiateDefaultModule());

    setLayout(outerLayout);
}
