/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "StateViewer.h"

#include <Components/FilterSelector/FilterSelector.h>
#include <Core/MessageBroker/MessageBroker.h>

StateViewerModule::StateViewerModule(QWidget* parent) : DefaultModule(parent)
{
    setupUi();
    defaultContextMenuSetup();
}

StateViewerModule::~StateViewerModule()
{
    getCore()->getMessageBroker()->unsubscribe(filter, this);
}

QWidget* StateViewerModule::toWidget() { return this; }

XmlObject StateViewerModule::toXmlObject()
{
    XmlObject obj(getName(ModuleId::STATEVIEWER));

    obj.addAttribute("filter", filter.toString());

    return obj;
}

void StateViewerModule::fromXmlObject(const XmlObject& xmlObject)
{
    if (xmlObject.getObjectName() == getName(ModuleId::STATEVIEWER))
    {
        auto filter = Filter::fromString(xmlObject.getAttribute("filter"));
        setFilter(filter);
    }
}

void StateViewerModule::setupUi()
{
    outerLayout = new QHBoxLayout;
    outerLayout->setContentsMargins(0, 0, 0, 0);
    outerLayout->setSpacing(0);

    for (auto labelText : StatesList::statesLabels)
    {
        QLabel* label = new QLabel;
        label->setText(labelText);
        label->setAlignment(Qt::AlignCenter);
        label->setContentsMargins(4, 4, 4, 4);
        outerLayout->addWidget(label);
    }

    setLayout(outerLayout);
}

void StateViewerModule::addCustomActionsToMenu()
{
    QAction* action = new QAction("Choose topic and field");
    connect(action, &QAction::triggered, this,
            &StateViewerModule::onConfigureClicked);

    addActionToMenu(action);
}

void StateViewerModule::onConfigureClicked()
{
    FilterSelector::selectFilter(
        filter, [this](const Filter& newFilter) { setFilter(newFilter); });
}

void StateViewerModule::setFilter(const Filter& newFilter)
{
    getCore()->getMessageBroker()->unsubscribe(filter, this);
    getCore()->getMessageBroker()->subscribe(
        newFilter, this,
        [this](const Message& message, const Filter& filter)
        { onMsgReceived(message); });
    filter = newFilter;
}

void StateViewerModule::onMsgReceived(const Message& msg)
{
    Field field = msg.getField("fmm_state");

    StatesList::State state =
        static_cast<StatesList::State>(field.getUnsignedInteger());

    if (state == StatesList::State::INVALID)
        return;

    if (state != currentState)
    {
        currentState = static_cast<StatesList::State>(state);

        QString baseStyle =
            "border-bottom-width:1px;border-left-width:1px;border-top-width:"
            "1px;border-radius:0;";
        QString currentStateStyle =
            "background-color:yellow; color:black;" + baseStyle;
        QString completedStyle = "background-color:green;" + baseStyle;
        QString errorStyle     = "background-color:red;" + baseStyle;

        int index  = StatesList::statesIndexes[currentState];
        auto label = outerLayout->itemAt(index)->widget();

        if (currentState == StatesList::State::INIT_ERROR)
            label->setStyleSheet(errorStyle);
        else
            label->setStyleSheet(currentStateStyle);

        // Set as completed every state before the current
        for (int i = index - 1; i >= 0; i--)
            outerLayout->itemAt(i)->widget()->setStyleSheet(completedStyle);

        // Reset every state after the current
        for (int i = index + 1; i < StatesList::statesLabels.count(); i++)
            outerLayout->itemAt(i)->widget()->setStyleSheet(baseStyle);

        auto tmp = outerLayout->itemAt(StatesList::statesLabels.count() - 1)
                       ->widget()
                       ->styleSheet();
        outerLayout->itemAt(StatesList::statesLabels.count() - 1)
            ->widget()
            ->setStyleSheet(tmp + "border-right-width:1px;");
    }
}
